@@ -30,7 +30,7 @@
 #include "xbus-core.h"
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: card_global.c 9916 2011-05-23 13:37:40Z tzafrir $";
+static const char rcsid[] = "$Id: card_global.c 10489 2012-03-15 17:35:57Z tzafrir $";
 
 DEF_PARM(charp,initdir, "/usr/share/dahdi", 0644, "The directory of card initialization scripts");
 
@@ -40,6 +40,36 @@ extern	int debug;
 
 /*---------------- GLOBAL PROC handling -----------------------------------*/
 
+static int send_magic_request(xbus_t *xbus,
+	unsigned unit, xportno_t portno, bool eoftx)
+{
+	xframe_t	*xframe;
+	xpacket_t	*pack;
+	reg_cmd_t	*reg_cmd;
+	int		ret;
+
+	/*
+	 * Zero length multibyte is legal and has special meaning for the
+	 * firmware:
+	 *   eoftx==1: Start sending us D-channel packets.
+	 *   eoftx==0: Stop sending us D-channel packets.
+	 */
+	XFRAME_NEW_CMD(xframe, pack, xbus, GLOBAL, REGISTER_REQUEST, unit);
+	reg_cmd = &RPACKET_FIELD(pack, GLOBAL, REGISTER_REQUEST, reg_cmd);
+	reg_cmd->bytes = 0;
+	reg_cmd->is_multibyte = 1;
+	reg_cmd->portnum = portno;
+	reg_cmd->eoframe = eoftx;
+	PORT_DBG(REGS, xbus, unit, portno, "Magic Packet (eoftx=%d)\n", eoftx);
+	if (debug & DBG_REGS)
+		dump_xframe(__func__, xbus, xframe, debug);
+	ret = send_cmd_frame(xbus, xframe);
+	if (ret < 0)
+		PORT_ERR(xbus, unit, portno,
+			"%s: failed sending xframe\n", __func__);
+	return ret;
+}
+
 static int parse_hexbyte(const char *buf)
 {
 	char		*endp;
@@ -155,8 +185,8 @@ static int execute_chip_command(xpd_t *xpd, const int argc, char *argv[])
 					addr_mode, argc - argno);
 			goto out;
 		}
-		ret = send_multibyte_request(xpd->xbus, xpd->addr.unit, portno,
-			addr_mode == 'm', NULL, 0);
+		ret = send_magic_request(xpd->xbus, xpd->addr.unit, portno,
+			addr_mode == 'm');
 		goto out;
 	}
 	/* Normal (non-Magic) register commands */
@@ -370,44 +400,6 @@ int xpp_register_request(xbus_t *xbus, xpd_t *xpd, xportno_t portno,
 	return ret;
 }
 
-int send_multibyte_request(xbus_t *xbus,
-	unsigned unit, xportno_t portno,
-	bool eoftx, byte *buf, unsigned len)
-{
-	xframe_t	*xframe;
-	xpacket_t	*pack;
-	reg_cmd_t	*reg_cmd;
-	int		ret;
-
-	/*
-	 * Zero length multibyte is legal and has special meaning for the
-	 * firmware:
-	 *   eoftx==1: Start sending us D-channel packets.
-	 *   eoftx==0: Stop sending us D-channel packets.
-	 */
-	if(len > MULTIBYTE_MAX_LEN) {
-		PORT_ERR(xbus, unit, portno, "%s: len=%d is too long. dropping.\n", __FUNCTION__, len);
-		return -EINVAL;
-	}
-	XFRAME_NEW_CMD(xframe, pack, xbus, GLOBAL, REGISTER_REQUEST, unit);
-	reg_cmd = &RPACKET_FIELD(pack, GLOBAL, REGISTER_REQUEST, reg_cmd);
-	reg_cmd->bytes = len;
-	reg_cmd->is_multibyte = 1;
-	reg_cmd->portnum = portno;
-	reg_cmd->eoframe = eoftx;
-	if(len > 0) {
-		memcpy(REG_XDATA(reg_cmd), (byte *)buf, len);
-	} else {
-		PORT_DBG(REGS, xbus, unit, portno, "Magic Packet (eoftx=%d)\n", eoftx);
-	}
-	if(debug & DBG_REGS)
-		dump_xframe(__FUNCTION__, xbus, xframe, debug);
-	ret = send_cmd_frame(xbus, xframe);
-	if(ret < 0)
-		PORT_ERR(xbus, unit, portno, "%s: failed sending xframe\n", __FUNCTION__);
-	return ret;
-}
-
 /*
  * The XPD parameter is totaly ignored by the driver and firmware as well.
  */
@@ -430,19 +422,37 @@ int send_multibyte_request(xbus_t *xbus,
 	return 0;
 }
 
-/* 0x23 */ HOSTCMD(GLOBAL, RESET_SYNC_COUNTERS)
+/*
+ * Wrapper for different types of xbus reset
+ */
+static int send_xbus_reset(xbus_t *xbus, uint8_t reset_mask)
 {
 	xframe_t	*xframe;
 	xpacket_t	*pack;
 
 	BUG_ON(!xbus);
-	//XBUS_DBG(SYNC, xbus, "\n");
-	XFRAME_NEW_CMD(xframe, pack, xbus, GLOBAL, RESET_SYNC_COUNTERS, 0);
-	RPACKET_FIELD(pack, GLOBAL, RESET_SYNC_COUNTERS, mask) = 0x10;
+	XFRAME_NEW_CMD(xframe, pack, xbus, GLOBAL, XBUS_RESET, 0);
+	RPACKET_FIELD(pack, GLOBAL, XBUS_RESET, mask) = reset_mask;
 	send_cmd_frame(xbus, xframe);
 	return 0;
 }
 
+/* 0x23 */ HOSTCMD(GLOBAL, RESET_SPI)
+{
+	XBUS_DBG(DEVICES, xbus, "Sending SPI reset\n");
+	/* toggle reset line */
+	send_xbus_reset(xbus, 0x04);
+	send_xbus_reset(xbus, 0x00);
+	return 0;
+}
+
+
+/* 0x23 */ HOSTCMD(GLOBAL, RESET_SYNC_COUNTERS)
+{
+	//XBUS_DBG(SYNC, xbus, "\n");
+	return send_xbus_reset(xbus, 0x10);
+}
+
 /*---------------- GLOBAL: Astribank Reply Handlers -----------------------*/
 
 HANDLER_DEF(GLOBAL, NULL_REPLY)
@@ -534,6 +544,7 @@ HANDLER_DEF(GLOBAL, AB_DESCRIPTION)	/* 0x08 */
 		list_add_tail(&card_desc->card_list, &worker->card_list);
 		spin_unlock_irqrestore(&worker->worker_lock, flags);
 	}
+	CALL_PROTO(GLOBAL, RESET_SPI, xbus, NULL);
 	if (!xbus_process_worker(xbus)) {
 		ret = -ENODEV;
 		goto out;
@@ -642,7 +653,6 @@ static void global_packet_dump(const char *msg, xpacket_t *pack)
 	DBG(GENERAL, "%s\n", msg);
 }
 
-#define	MAX_ENV_STR	40
 #define	MAX_PATH_STR	128
 
 int run_initialize_registers(xpd_t *xpd)
@@ -651,6 +661,7 @@ int run_initialize_registers(xpd_t *xpd)
 	xbus_t	*xbus;
 	char	busstr[MAX_ENV_STR];
 	char	busnumstr[MAX_ENV_STR];
+	char	modelstr[MAX_ENV_STR];
 	char	unitstr[MAX_ENV_STR];
 	char	subunitsstr[MAX_ENV_STR];
 	char	typestr[MAX_ENV_STR];
@@ -668,6 +679,7 @@ int run_initialize_registers(xpd_t *xpd)
 	char	*envp[] = {
 		busstr,
 		busnumstr,
+		modelstr,
 		unitstr,
 		subunitsstr,
 		typestr,
@@ -703,6 +715,7 @@ int run_initialize_registers(xpd_t *xpd)
 	}
 	snprintf(busstr, MAX_ENV_STR, "XBUS_NAME=%s", xbus->busname);
 	snprintf(busnumstr, MAX_ENV_STR, "XBUS_NUMBER=%d", xbus->num);
+	snprintf(modelstr, MAX_ENV_STR, "XBUS_MODEL_STRING=%s", xbus->transport.model_string);
 	snprintf(unitstr, MAX_ENV_STR, "UNIT_NUMBER=%d", xpd->addr.unit);
 	snprintf(typestr, MAX_ENV_STR, "UNIT_TYPE=%d", xpd->type);
 	snprintf(subunitsstr, MAX_ENV_STR, "UNIT_SUBUNITS=%d", xpd->subunits);
@@ -750,4 +763,3 @@ err:
 EXPORT_SYMBOL(sync_mode_name);
 EXPORT_SYMBOL(run_initialize_registers);
 EXPORT_SYMBOL(xpp_register_request);
-EXPORT_SYMBOL(send_multibyte_request);
