@@ -33,7 +33,7 @@
 #include "dahdi_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_pri.c 10053 2011-07-20 16:49:07Z tzafrir $";
+static const char rcsid[] = "$Id: card_pri.c 10347 2011-11-29 23:38:30Z tzafrir $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(uint, poll_interval, 500, 0644, "Poll channel state interval in milliseconds (0 - disable)");
@@ -80,6 +80,7 @@ static int pri_shutdown(struct dahdi_span *span);
 static int pri_rbsbits(struct dahdi_chan *chan, int bits);
 static int pri_lineconfig(xpd_t *xpd, int lineconfig);
 static void send_idlebits(xpd_t *xpd, bool saveold);
+static int apply_pri_protocol(xpd_t *xpd);
 
 #define	PROC_REGISTER_FNAME	"slics"
 
@@ -90,17 +91,29 @@ enum pri_protocol {
 	PRI_PROTO_J1 	= 3
 };
 
+static const char *protocol_names[] = {
+	[PRI_PROTO_0] = "??",	/* unknown */
+	[PRI_PROTO_E1] = "E1",
+	[PRI_PROTO_T1] = "T1",
+	[PRI_PROTO_J1] = "J1"
+};
+
 static const char *pri_protocol_name(enum pri_protocol pri_protocol)
 {
-	static const char *protocol_names[] = {
-		[PRI_PROTO_0] = "??",	/* unknown */
-		[PRI_PROTO_E1] = "E1",
-		[PRI_PROTO_T1] = "T1",
-		[PRI_PROTO_J1] = "J1"
-		};
 	return protocol_names[pri_protocol];
 }
 
+static enum pri_protocol pri_protocol_bystr(const char *spantype)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(protocol_names); i++) {
+		if (strcasecmp(protocol_names[i], spantype) == 0)
+			return i;
+	}
+	return PRI_PROTO_0;
+}
+
 static int pri_num_channels(enum pri_protocol pri_protocol)
 {
 	static int num_channels[] = {
@@ -551,12 +564,14 @@ static int set_pri_proto(xpd_t *xpd, enum pri_protocol set_proto)
 	unsigned int		dchan_num;
 	int			default_lineconfig = 0;
 	int			ret;
+	struct phonedev		*phonedev;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
-	if(SPAN_REGISTERED(xpd)) {
-		XPD_NOTICE(xpd, "Registered as span %d. Cannot do setup pri protocol (%s)\n",
-			PHONEDEV(xpd).span.spanno, __FUNCTION__);
+	phonedev = &PHONEDEV(xpd);
+	if (test_bit(DAHDI_FLAGBIT_REGISTERED, &phonedev->span.flags)) {
+		XPD_NOTICE(xpd, "%s: %s already assigned as span %d\n",
+			__func__, phonedev->span.name, phonedev->span.spanno);
 		return -EBUSY;
 	}
 	if(priv->pri_protocol != PRI_PROTO_0) {
@@ -596,8 +611,8 @@ static int set_pri_proto(xpd_t *xpd, enum pri_protocol set_proto)
 	}
 	priv->pri_protocol = set_proto;
 	priv->is_cas = -1;
-	PHONEDEV(xpd).channels = pri_num_channels(set_proto);
-	PHONEDEV(xpd).offhook_state = BITMASK(PHONEDEV(xpd).channels);
+	phonedev->channels = pri_num_channels(set_proto);
+	phonedev->offhook_state = BITMASK(phonedev->channels);
 	CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 	priv->deflaw = deflaw;
 	priv->dchan_num = dchan_num;
@@ -605,7 +620,7 @@ static int set_pri_proto(xpd_t *xpd, enum pri_protocol set_proto)
 	xpd->type_name = type_name(priv->pri_protocol);
 	XPD_DBG(GENERAL, xpd, "%s, channels=%d, dchan_num=%d, deflaw=%d\n",
 			pri_protocol_name(set_proto),
-			PHONEDEV(xpd).channels,
+			phonedev->channels,
 			priv->dchan_num,
 			priv->deflaw
 			);
@@ -618,7 +633,7 @@ static int set_pri_proto(xpd_t *xpd, enum pri_protocol set_proto)
 		XPD_NOTICE(xpd, "Failed setting PRI default line config\n");
 		return ret;
 	}
-	return 0;
+	return apply_pri_protocol(xpd);
 }
 
 static void dahdi_update_syncsrc(xpd_t *xpd)
@@ -1019,6 +1034,17 @@ static int pri_lineconfig(xpd_t *xpd, int lineconfig)
 	}
 #endif
 	if(force_cas) {
+		if(priv->pri_protocol == PRI_PROTO_E1) {
+			int	rs1 = 0x0B;
+
+			/*
+			 * Set correct X1-X3 bits in the E1 CAS MFAS
+			 * They are unused in E1 and should be 1
+			 */
+			XPD_DBG(GENERAL, xpd, "%s: rs1(0x%02X) = 0x%02X\n",
+				__FUNCTION__, REG_RS1_E, rs1);
+			write_subunit(xpd, REG_RS1_E, rs1);
+		}
 		xsp |= REG_XSP_E_CASEN;	/* Same as REG_FMR5_T_EIBR for T1 */
 	}
 	XPD_DBG(GENERAL, xpd, "%s: xsp(0x%02X) = 0x%02X\n", __FUNCTION__, REG_XSP_E, xsp);
@@ -1029,6 +1055,17 @@ bad_lineconfig:
 	return -EINVAL;
 }
 
+static int pri_set_spantype(struct dahdi_span *span, const char *spantype)
+{
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
+	enum pri_protocol set_proto = PRI_PROTO_0;
+
+	XPD_INFO(xpd, "%s: %s\n", __func__, spantype);
+	set_proto = pri_protocol_bystr(spantype);
+	return set_pri_proto(xpd, set_proto);
+}
+
 /*
  * Called only for 'span' keyword in /etc/dahdi/system.conf
  */
@@ -1204,6 +1241,7 @@ static int pri_audio_notify(struct dahdi_chan *chan, int on)
 
 static const struct dahdi_span_ops PRI_span_ops = {
 	.owner = THIS_MODULE,
+	.set_spantype = pri_set_spantype,
 	.spanconfig = pri_spanconfig,
 	.chanconfig = pri_chanconfig,
 	.startup = pri_startup,
@@ -1215,6 +1253,7 @@ static const struct dahdi_span_ops PRI_span_ops = {
 	.maint = xpp_maint,
 	.echocan_create = xpp_echocan_create,
 	.echocan_name = xpp_echocan_name,
+	.assigned = xpp_span_assigned,
 #ifdef	DAHDI_SYNC_TICK
 	.sync_tick = dahdi_sync_tick,
 #endif
@@ -1227,7 +1266,7 @@ static const struct dahdi_span_ops PRI_span_ops = {
 #endif
 };
 
-static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
+static int apply_pri_protocol(xpd_t *xpd)
 {
 	xbus_t			*xbus;
 	struct PRI_priv_data	*priv;
@@ -1237,15 +1276,7 @@ static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 	xbus = xpd->xbus;
 	priv = xpd->priv;
 	BUG_ON(!xbus);
-	XPD_DBG(GENERAL, xpd, "%s (proto=%s, channels=%d, deflaw=%d)\n",
-		(on)?"on":"off",
-		pri_protocol_name(priv->pri_protocol),
-		PHONEDEV(xpd).channels,
-		priv->deflaw);
-	if(!on) {
-		/* Nothing to do yet */
-		return 0;
-	}
+	XPD_DBG(GENERAL, xpd, "\n");
 	PHONEDEV(xpd).span.spantype = pri_protocol_name(priv->pri_protocol);
 	PHONEDEV(xpd).span.linecompat = pri_linecompat(priv->pri_protocol);
 	PHONEDEV(xpd).span.deflaw = priv->deflaw;
@@ -1268,9 +1299,32 @@ static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 	}
 	PHONEDEV(xpd).offhook_state = PHONEDEV(xpd).wanted_pcm_mask;
 	PHONEDEV(xpd).span.ops = &PRI_span_ops;
+	PHONEDEV(xpd).span.channels = PHONEDEV(xpd).channels;
+	xpd_set_spanname(xpd);
 	return 0;
 }
 
+static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
+{
+	xbus_t			*xbus;
+	struct PRI_priv_data	*priv;
+
+	BUG_ON(!xpd);
+	xbus = xpd->xbus;
+	priv = xpd->priv;
+	BUG_ON(!xbus);
+	XPD_DBG(GENERAL, xpd, "%s (proto=%s, channels=%d, deflaw=%d)\n",
+		(on)?"on":"off",
+		pri_protocol_name(priv->pri_protocol),
+		PHONEDEV(xpd).channels,
+		priv->deflaw);
+	if(!on) {
+		/* Nothing to do yet */
+		return 0;
+	}
+	return apply_pri_protocol(xpd);
+}
+
 static int PRI_card_dahdi_postregistration(xpd_t *xpd, bool on)
 {
 	xbus_t			*xbus;
@@ -1424,14 +1478,17 @@ static int PRI_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 		return -ENODEV;
 	chan = XPD_CHAN(xpd, pos);
 	switch (cmd) {
+		/*
+		 * Asterisk may send FXS type ioctl()'s to us:
+		 *   - Some are sent to everybody (DAHDI_TONEDETECT)
+		 *   - Some are sent because we may be in CAS mode
+		 *     (FXS signalling)
+		 * Ignore them.
+		 */
 		case DAHDI_TONEDETECT:
-			/*
-			 * Asterisk call all span types with this (FXS specific)
-			 * call. Silently ignore it.
-			 */
 			LINE_DBG(SIGNAL, xpd, pos, "PRI: TONEDETECT (%s)\n",
 				(chan->flags & DAHDI_FLAG_AUDIO) ?
-					"AUDIO" : "SILENCE");
+				"AUDIO" : "SILENCE");
 			return -ENOTTY;
 		case DAHDI_ONHOOKTRANSFER:
 			LINE_DBG(SIGNAL, xpd, pos, "PRI: ONHOOKTRANSFER\n");
@@ -1442,6 +1499,10 @@ static int PRI_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 		case DAHDI_VMWI_CONFIG:
 			LINE_DBG(SIGNAL, xpd, pos, "PRI: VMWI_CONFIG\n");
 			return -ENOTTY;
+		case DAHDI_SETPOLARITY:
+			LINE_DBG(SIGNAL, xpd, pos, "PRI: SETPOLARITY\n");
+			return -ENOTTY;
+		/* report on really bad ioctl()'s */
 		default:
 			report_bad_ioctl(THIS_MODULE->name, xpd, pos, cmd);
 			return -ENOTTY;
@@ -2210,19 +2271,20 @@ static DEVICE_ATTR_WRITER(pri_protocol_store, dev, buf, count)
 	BUG_ON(!dev);
 	xpd = dev_to_xpd(dev);
 	XPD_DBG(GENERAL, xpd, "%s\n", buf);
-	if(!xpd)
+	if (!xpd)
 		return -ENODEV;
-	if((i = strcspn(buf, " \r\n")) != 2) {
+	i = strcspn(buf, " \r\n");
+	if (i != 2) {
 		XPD_NOTICE(xpd,
 			"Protocol name '%s' has %d characters (should be 2). Ignored.\n",
 			buf, i);
 		return -EINVAL;
 	}
-	if(strnicmp(buf, "E1", 2) == 0)
+	if (strnicmp(buf, "E1", 2) == 0)
 		new_protocol = PRI_PROTO_E1;
-	else if(strnicmp(buf, "T1", 2) == 0)
+	else if (strnicmp(buf, "T1", 2) == 0)
 		new_protocol = PRI_PROTO_T1;
-	else if(strnicmp(buf, "J1", 2) == 0)
+	else if (strnicmp(buf, "J1", 2) == 0)
 		new_protocol = PRI_PROTO_J1;
 	else {
 		XPD_NOTICE(xpd,
@@ -2234,7 +2296,8 @@ static DEVICE_ATTR_WRITER(pri_protocol_store, dev, buf, count)
 	return (ret < 0) ? ret : count;
 }
 
-static	DEVICE_ATTR(pri_protocol, S_IRUGO | S_IWUSR, pri_protocol_show, pri_protocol_store);
+static	DEVICE_ATTR(pri_protocol, S_IRUGO | S_IWUSR, pri_protocol_show,
+		pri_protocol_store);
 
 static DEVICE_ATTR_READER(pri_localloop_show, dev, buf)
 {
