@@ -40,7 +40,7 @@
 #include "xbus-core.h"
 #include "dahdi_debug.h"
 
-static const char rcsid[] = "$Id: xbus-sysfs.c 9993 2011-06-28 18:23:00Z tzafrir $";
+static const char rcsid[] = "$Id: xbus-sysfs.c 10334 2011-11-10 16:56:29Z tzafrir $";
 
 /* Command line parameters */
 extern int debug;
@@ -438,16 +438,6 @@ static void astribank_release(struct device *dev)
 	xbus_free(xbus);
 }
 
-static void toplevel_release(struct device *dev)
-{
-	NOTICE("%s\n", __func__);
-}
-
-static struct device toplevel_device = {
-	.release	= toplevel_release,
-	/* No Parent */
-};
-
 static struct bus_type toplevel_bus_type = {
 	.name           = "astribanks",
 	.match          = astribank_match,
@@ -623,6 +613,10 @@ static DEVICE_ATTR_READER(span_show, dev, buf)
 	return len;
 }
 
+/*
+ * For backward compatibility with old dahdi-tools
+ * Remove after dahdi_registration is upgraded
+ */
 static DEVICE_ATTR_WRITER(span_store, dev, buf, count)
 {
 	xpd_t		*xpd;
@@ -631,19 +625,34 @@ static DEVICE_ATTR_WRITER(span_store, dev, buf, count)
 
 	BUG_ON(!dev);
 	xpd = dev_to_xpd(dev);
-	if(!xpd)
+	if (!xpd)
 		return -ENODEV;
 	ret = sscanf(buf, "%d", &dahdi_reg);
-	if(ret != 1)
+	if (ret != 1)
 		return -EINVAL;
-	if(!XBUS_IS(xpd->xbus, READY))
+	if (!XBUS_IS(xpd->xbus, READY))
 		return -ENODEV;
-	XPD_DBG(GENERAL, xpd, "%s\n", (dahdi_reg) ? "register" : "unregister");
-	if(dahdi_reg)
-		ret = dahdi_register_xpd(xpd);
-	else
-		ret = dahdi_unregister_xpd(xpd);
-	return (ret < 0) ? ret : count;
+	XPD_DBG(DEVICES, xpd,
+		"%s -- deprecated (should use pinned-spans)\n",
+		(dahdi_reg) ? "register" : "unregister");
+	if (xbus_is_registered(xpd->xbus)) {
+		if (dahdi_reg) {
+			XPD_DBG(DEVICES, xpd,
+				"already registered %s. Ignored.\n",
+				xpd->xbus->busname);
+		} else {
+			xbus_unregister_dahdi_device(xpd->xbus);
+		}
+	} else {
+		if (!dahdi_reg) {
+			XPD_DBG(DEVICES, xpd,
+				"already unregistered %s. Ignored.\n",
+				xpd->xbus->busname);
+		} else {
+			xbus_register_dahdi_device(xpd->xbus);
+		}
+	}
+	return count;
 }
 
 static DEVICE_ATTR_READER(type_show, dev, buf)
@@ -964,7 +973,7 @@ int xbus_sysfs_create(xbus_t *xbus)
 	astribank = &xbus->astribank;
 	XBUS_DBG(DEVICES, xbus, "\n");
 	astribank->bus = &toplevel_bus_type;
-	astribank->parent = &toplevel_device;
+	astribank->parent = xbus->transport.transport_device;
 	dev_set_name(astribank, "xbus-%02d", xbus->num);
 	dev_set_drvdata(astribank, xbus);
 	astribank->release = astribank_release;
@@ -981,16 +990,10 @@ int __init xpp_driver_init(void)
 	int	ret;
 
 	DBG(DEVICES, "SYSFS\n");
-	dev_set_name(&toplevel_device, "astribanks");
-	ret = device_register(&toplevel_device);
-	if (ret) {
-		ERR("%s: toplevel device_register failed: %d\n", __func__, ret);
-		goto failed_toplevel;
-	}
 	if((ret = bus_register(&toplevel_bus_type)) < 0) {
 		ERR("%s: bus_register(%s) failed. Error number %d",
 			__FUNCTION__, toplevel_bus_type.name, ret);
-		goto failed_bus;
+		goto failed_toplevel;
 	}
 	if((ret = driver_register(&xpp_driver)) < 0) {
 		ERR("%s: driver_register(%s) failed. Error number %d",
@@ -1007,8 +1010,6 @@ failed_xpd_bus:
 	driver_unregister(&xpp_driver);
 failed_xpp_driver:
 	bus_unregister(&toplevel_bus_type);
-failed_bus:
-	device_unregister(&toplevel_device);
 failed_toplevel:
 	return ret;
 }
@@ -1019,7 +1020,6 @@ void xpp_driver_exit(void)
 	bus_unregister(&xpd_type);
 	driver_unregister(&xpp_driver);
 	bus_unregister(&toplevel_bus_type);
-	device_unregister(&toplevel_device);
 }
 
 EXPORT_SYMBOL(xpd_driver_register);
