@@ -42,7 +42,7 @@
 #endif
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xpp_usb.c 9929 2011-06-02 20:00:36Z sruffell $";
+static const char rcsid[] = "$Id: xpp_usb.c 10300 2011-11-02 14:08:12Z tzafrir $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(int, usb1, 0, 0644, "Allow using USB 1.1 interfaces");
@@ -737,6 +737,12 @@ static int xusb_probe(struct usb_interface *interface, const struct usb_device_i
 		retval = -ENOMEM;
 		goto probe_failed;
 	}
+	snprintf(xbus->transport.model_string,
+		ARRAY_SIZE(xbus->transport.model_string),
+		"usb:%04x/%04x/%x",
+		udev->descriptor.idVendor,
+		udev->descriptor.idProduct,
+		udev->descriptor.bcdDevice);
 	spin_lock_irqsave(&xusb_lock, flags);
 	for(i = 0; i < MAX_BUSES; i++) {
 		if(xusb_array[i] == NULL)
